import fs from 'fs';
import ms from 'ms';
import axios from 'axios';
class Request {
    async send(url, options) {
        try {
            const response = await axios({ url, ...options });
            return response.data;
        }
        catch (error) {
            return Promise.reject(error);
        }
    }
    async retryCall(url, options, retryOptions) {
        const { maxRetry = 3, interval = ms('5s') } = retryOptions || {};
        let attempt = 1;
        return new Promise((resolve, reject) => {
            this.send(url, options)
                .then((response) => resolve(response))
                .catch(() => {
                const retryIntervalId = setInterval(async () => {
                    attempt += 1;
                    const responseRetry = await this.send(url, options).catch((err) => ({ error: true, message: err.message }));
                    if (responseRetry.error && attempt >= maxRetry) {
                        clearInterval(retryIntervalId);
                        reject(new Error(responseRetry.message));
                    }
                    else if (!responseRetry.error) {
                        clearInterval(retryIntervalId);
                        resolve(responseRetry);
                    }
                }, interval);
            });
        });
    }
    removeStartEndBar(text) {
        return text.replace(/^\/|\/+$/g, '');
    }
    buildUrl(url, { queryParams, endPoint }) {
        let newUrl = this.removeStartEndBar(url);
        if (endPoint)
            newUrl = `${newUrl}/${this.removeStartEndBar(endPoint)}`;
        if (queryParams && !!Object.keys(queryParams).length) {
            newUrl = `${newUrl}?${this.getQueryParams(queryParams)}`;
        }
        return newUrl;
    }
    getQueryParams(data = {}) {
        const searchParams = new URLSearchParams(Object.entries(data));
        return searchParams.toString();
    }
    async post(url, data = {}, options = {}) {
        return this.send(url, { ...options, data, method: 'POST' });
    }
    async get(url, queryParams = {}, options = {}) {
        const encodedUrl = this.buildUrl(url, { queryParams });
        return this.send(encodedUrl, { ...options, method: 'GET' });
    }
    async put(url, data = {}, options = {}) {
        return this.send(url, { ...options, data, method: 'PUT' });
    }
    async delete(url, data = {}, options = {}) {
        return this.send(url, { ...options, data, method: 'DELETE' });
    }
    getAllParams(request) {
        return { ...request.params, ...request.body, ...request.query };
    }
    handleResponseError(response, error) {
        if (error instanceof Error) {
            response.status(500).send({ error: error.message });
        }
        else {
            response.status(500).send({ error: 'UNEXPECTED_ERROR' });
        }
    }
    async downloadFile(url, outputPath) {
        const fileName = url.split('/').pop();
        const result = {
            fileName,
            error: false,
            message: '',
        };
        try {
            const writer = fs.createWriteStream(outputPath);
            return await new Promise((resolve, reject) => {
                axios({ method: 'get', url, responseType: 'stream' }).then((axiosResponse) => {
                    axiosResponse.data.pipe(writer);
                    writer.on('finish', () => {
                        resolve(result);
                    });
                    writer.on('error', (error) => {
                        result.error = true;
                        result.message = error.message;
                        reject(result);
                    });
                }).catch((err) => {
                    result.error = true;
                    result.message = err.message;
                    return Promise.reject(result);
                });
            });
        }
        catch (error) {
            result.error = true;
            result.message = error.message;
            return Promise.reject(result);
        }
    }
}
const instance = new Request();
export { instance as Request };
export default instance;
